<?php

/**
 * @file Drupal\popup_message\EventSubscriber\PopupMessageSubscriber
 */
namespace Drupal\popup_message\EventSubscriber;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Render\AttachmentsInterface;
use Drupal\Component\Utility\Unicode;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class PopupMessageSubscriber
 * @package Drupal\popup_message\EventSubscriber
 */
class PopupMessageSubscriber implements EventSubscriberInterface {

  /**
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * @var
   */
  protected $path;

  /**
   * PopupMessageSubscriber constructor.
   */
  public function __construct() {
    $this->config = \Drupal::configFactory()->get('popup_message.settings');
    $this->path = ltrim(\Drupal::service('path.current')->getPath(), '/');
  }

  /**
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   */
  public function showPopupMessage(FilterResponseEvent $event) {
    // Check permissions to display message.

    $response = $event->getResponse();

    if (!$response instanceof AttachmentsInterface) {
      return;
    }

    $permission = \Drupal::currentUser()
      ->hasPermission('display popup message');
    // Get status: enabled/disabled.
    $status = $this->config->get('enable');

    // Allow other modules to modiffy permissions.
    \Drupal::moduleHandler()->alter('popup_message_permission', $permission);

    // Get popup message visibility settings.
     $visibility = $this->config->get('visibility') ? $this->config->get('visibility') : POPUP_MESSAGE_VISIBILITY_NOTLISTED;

    // Get popup message visibility pages settings.
     $visibility_pages = $this->config->get('visibility_pages') ? $this->config->get('visibility_pages') : '';

    // Predefine value.
    $page_match = TRUE;

    // Limited visibility popup message must list at least one page.
    if ($visibility == POPUP_MESSAGE_VISIBILITY_LISTED && empty($visibility_pages)) {
      $status = FALSE;
    }

    // Match path if necessary.
    if ($visibility_pages && $status) {
      // Convert path to lowercase. This allows comparison of the same path
      // with different case. Ex: /Page, /page, /PAGE.
      $pages = Unicode::strtolower($visibility_pages);
      if ($visibility < POPUP_MESSAGE_VISIBILITY_MAX) {
        // Convert the Drupal path to lowercase.
        $path = Unicode::strtolower($this->path);
        // Compare the lowercase internal and lowercase path alias (if any).
        $page_match = \Drupal::service('path.matcher')->matchPath($path, $pages);

        // When $visibility has a value of 0 (POPUP_MESSAGE_VISIBILITY_NOTLISTED),
        // the popup message is displayed on all pages except those listed in
        // $visibility_pages.
        // When set to 1 (POPUP_MESSAGE_VISIBILITY_LISTED), it is displayed only
        // on those
        // pages listed in $visibility_pages.
        $page_match = !($visibility xor $page_match);
      }
      else {
        $page_match = FALSE;
      }
    }

    // Predefine value.
    $cookie_exist = FALSE;

    // Check if cookie exist.
    if(isset($_COOKIE['popup_message_displayed'])) {
      $cookie_exist = TRUE;
    }

    // Get cookie status: enabled/disabled.
    $cookie_enable = $this->config->get('check_cookie');

    if ($permission && $status && $page_match && !$cookie_exist && $cookie_enable) {
      $message_title = Xss::filter($this->config->get('title'));
      $message_body_variable = $this->config->get('body');
      $message_body = check_markup(
        $message_body_variable['value'],
        ($message_body_variable['format'] ? $message_body_variable['format'] : filter_default_format()),
        FALSE
      );

      $popup_message_parameters = array(
        'title' => $message_title,
        'body' => $message_body,
        'check_cookie' => $this->config->get('check_cookie') ? $this->config->get('check_cookie') : 0,
        'width' => $this->config->get('width') ? $this->config->get('width') : 300,
        'height' => $this->config->get('height') ? $this->config->get('height') : 300,
        'delay' => $this->config->get('delay') ? $this->config->get('delay') : 0,
      );
      // Allow other modules to modiffy message parameters.
      \Drupal::moduleHandler()
        ->alter('popup_message_perameters', $popup_message_parameters);

      if ($message_title && $message_body) {
        $attachments = $response->getAttachments();
        $attachments['library'][] = 'popup_message/popup_message_style';
        $attachments['drupalSettings']['popupMessage'] = $popup_message_parameters;

        $response->setAttachments($attachments);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = array('showPopupMessage', 100);

    return $events;
  }
}
